@@ -830,8 +830,12 @@ class EAObject(MutableMapping, metaclass=EAMeta):
                 try:
                     resolved = self._resolve_attr(k)
                 except AttributeError:
-                    # Keep track of unrecognized attributes to give a more informative exception.
-                    unrecognized.append(k)
+                    # See if we can set the attribute, such as through a property.
+                    try:
+                        setattr(self, k, v)
+                    except AttributeError:
+                        # Keep track of unrecognized attributes to give a more informative exception.
+                        unrecognized.append(k)
                 else:
                     if resolved in attr_to_alias:
                         raise ValueError(f'Multiple aliases given for {resolved}: {attr_to_alias[resolved]}, {k}')
@@ -888,7 +892,14 @@ class EAObject(MutableMapping, metaclass=EAMeta):
         return f'{type(self).__name__}({", ".join(f"{k}={v}" for k, v in self.items())})'
 
     def __setattr__(self, attr: str, value: EAValue) -> None:
-        resolved = self._resolve_attr(attr)
+        try:
+            resolved = self._resolve_attr(attr)
+        except AttributeError:
+            if hasattr(self, attr):
+                object.__setattr__(self, attr, value)
+                return
+            else:
+                raise
         if value is None:
             if resolved in self.__dict__:
                 # Be consistent about not including attributes with value None in self.__dict__.
